#!/bin/bash

# Joseph Polizzotto
# UC Berkeley
# 510-642-0329
# Version 1.6.3
# Instructions: 1) From a directory containing DOCX file(s) to convert, open a Git bash window and enter the path to the script. 2) Enter the name of the stylesheet you wish to use 3) enter the ISO value of the secondary language in the document (optional) and 4) Press ENTER.

stylesheet=$1
language=$2
TIMESTAMP=`date "+%m-%d-%Y %H:%M"`

for x in ./*.docx; do

        basePath=${x%.*}
        baseName=${basePath##*/}

cp  "$x" ./"$baseName"_edited.docx 2> /dev/null

rm ./~*.docx 2> /dev/null

done

for x in ./*_edited.docx; do

        basePath=${x%.*}
        baseName=${basePath##*/}
		name=${x%_*}

# Check if DOCX files exist in current working directory. If not, exit the script.

if [[ ! -f "$x" ]] ; then	
	echo -e "\n\e[0;31mDOCX files are not located in this directory. Exiting...\e[0m"
exit

fi

# Check if Git is installed (which includes the dependencies needed in this script). if not, exit the script.

if ! [ -x "$(command -v git)" ]; then
  echo -e "\n\e[0;31mError: git (https://gitforwindows.org) is not installed. Exiting...\e[0m" >&2
  exit 1
fi

if [ $# -eq 0 ];
  then
    echo -e "\n\e[0;31mNo stylesheet name supplied. Exiting...\e[0m" >&2
  exit 1
fi

if [ ! -f "/c/stylesheets/"$stylesheet".css" ]; then
    echo -e "\n\e[0;31mCould not locate \e[35m"$stylesheet".css. \e[0;31mPlace the stylesheet in\e[0m \e[44m/c/stylesheets/\e[0;31m and run the script again. Exiting...\e[0m">&2
  exit 1
fi

# Check if Nu HTML Checker is installed. if not, exit the script.

if [ ! -f  /c/vnu-runtime-image/bin/vnu.bat ]; then
  echo -e "\n\e[31mError: Nu HTML Checker (https://validator.github.io/validator) was not found. Exiting....\e[0m'" >&2
  exit 1
fi

# Remove _HTML ending from filenames

mv "$x" "`echo $x | sed 's/_HTML//'`"

done

for x in ./*_edited.docx; do

        basePath=${x%.*}
        baseName=${basePath##*/}
		name=${x%_*}
		# name=$(echo "$x" | cut -f 1 -d '_')

# Create Directories

if [ ! -d ./"${x%.*}" ]; then

mkdir "${x%.*}"

fi
  
if [ ! -d ./Converted-DOCX-HTML ]; then

mkdir Converted-DOCX-HTML

fi

if [ ! -d ./Converted-DOCX-HTML/log.txt ]; then

touch ./Converted-DOCX-HTML/log.txt 

fi

done

for x in ./*_edited.docx; do

        basePath=${x%.*}
        baseName=${basePath##*/}
		name=${baseName%???????}


# Create HTML File

		echo -e "\nCreating a copy of \e[35m"$name".docx\e[0m..."
		
		echo -e "\nConverting \e[35m"$name".docx\e[0m to \e[32m"$name".html\e[0m..."
		
        pandoc "$x" -s -t html5 --toc --toc-depth=3 -M document-css=false -H /c/stylesheets/"$stylesheet".css --metadata pagetitle="$baseName" --mathjax --extract-media ./"$baseName" -o ./"$baseName"/"$baseName".html
			
# Add Language Text to HTML

		sed -i 's/lang=""/lang="en"/g' ./"$baseName"/"$baseName".html
		
# Clean up Invalid HTML
	
		sed -i 's/ type="text\/javascript"//g' ./"$baseName"/"$baseName".html
	
		sed -i 's/ type="text\/css"//g' ./"$baseName"/"$baseName".html

# New in 1.6.3
		
# Clean up Path to Images

		 sed -i 's/<img\ src=".*\\media\\/<img\ src=".\/media\//g' ./"$baseName"/"$baseName".html
		
#
		
# Add an ARIA main <main> landmark for the main region of the document.	

		if grep -qi "</nav>" ./"$baseName"/"$baseName".html > /dev/null ; then sed -zi 's/<\/nav>\n/<\/nav>\n<main>\n/g' ./"$baseName"/"$baseName".html ; else sed -zi 's/<\/head>\n<body>/<\/head>\n<body>\n<main>/g' ./"$baseName"/"$baseName".html ; fi		
	
# Add a closing element for the ARIA main </main> region of the document.

		sed -zi 's/<\/body>/<\/main>\n<\/body>/g' ./"$baseName"/"$baseName".html

# Remove ARIA role for the NAV region in the document (role="doc-toc").

		sed -i 's/<nav id="TOC" role="doc-toc">/<nav id="TOC">/g' ./"$baseName"/"$baseName".html
		
# Add an ARIA landmark in the document <footer> and add text about how to get in touch with the Alternative Media Unit of DSP.
		
		sed -zi "s/<\/main>\n/<\/main>\n<footer>\n<p role=\"contentinfo\">This document was created by the Alternative Media Unit of the Disabled Students' Program at UC Berkeley. For questions or concerns about this document, please contact us at dspamc@berkeley.edu.<\/p>\n<\/footer>\n/g" ./"$baseName"/"$baseName".html
		
# Perform Find and Replace in HTML for text that has dotted underline formatting ($$$)
	
		sed -i 's/\$\$\$/<span class="dotted">/g' ./"$baseName"/"$baseName".html
		
# Perform Find and Replace in HTML for text that has dashed underline formatting (^^^)
		
		sed -i 's/\^\^\^/<span class="dashed">/g' ./"$baseName"/"$baseName".html
		
# Perform Find and Replace in HTML for text that has double underline formatting (+++)
		
		sed -i 's/+++/<span class="doubleunderline">/g' ./"$baseName"/"$baseName".html
		
# Perform Find and Replace in HTML for text that has highlight formatting (~~~)
		
		sed -i 's/~~~/<mark>/g' ./"$baseName"/"$baseName".html
		
# Perform Find and Replace in HTML for text that has closing tag for highlight formatting (***)
		
		sed -i 's/\*\*\*/<\/mark>/g' ./"$baseName"/"$baseName".html
		
# Perform Find and Replace in HTML for text that has double strikethrough formatting (***)
		
		sed -i 's/???/<del style="text-decoration-style: double;">/g' ./"$baseName"/"$baseName".html

# Perform Find and Replace in HTML for text that has closing tag for double strikethrough formatting (!!!)

		sed -i 's/!!!/<\/del>/g' ./"$baseName"/"$baseName".html

# Change Footnote Opening Text from "Begin Footnote:" to "Footnote Begin:"

		sed -i 's/<p>Begin Footnote:<\/p>/<p>Footnote Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Footnote Opening if not capitalized

		sed -i 's/<p>Footnote begin:<\/p>/<p>Footnote Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Opening Text from "Begin Secondary Text:" to "Secondary Text Begin:"
		
		sed -i 's/<p>Begin Secondary Text:<\/p>/<p>Secondary Text Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text if not capitalized
		
		sed -i 's/<p>Secondary text begin:<\/p>/<p>Secondary Text Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Opening Text from "Begin Description:" to "Description Begin:"
		
		sed -i 's/<p>Begin Description:<\/p>/<p>Description Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Opening Text if not capitalized
		
		sed -i 's/<p>Description begin:<\/p>/<p>Description Begin:<\/p>/g' ./"$baseName"/"$baseName".html

# Change Footnote Opening Text from "Begin Footnote" to "Footnote Begin:"

		sed -i 's/<p>Begin Footnote<\/p>/<p>Footnote Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Opening Text from "Begin Secondary Text" to "Secondary Text Begin:"
		
		sed -i 's/<p>Begin Secondary Text<\/p>/<p>Secondary Text Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Opening Text from "Begin Description" to "Description Begin:"
		
		sed -i 's/<p>Begin Description<\/p>/<p>Description Begin:<\/p>/g' ./"$baseName"/"$baseName".html

# Change Footnote Opening Text from "Footnote Begin" to "Footnote Begin:"

		sed -i 's/<p>Footnote Begin<\/p>/<p>Footnote Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Opening Text from "Secondary Text Begin" to "Secondary Text Begin:"
		
		sed -i 's/<p>Secondary Text Begin<\/p>/<p>Secondary Text Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Text Opening Text from "Description Begin" to "Description Begin:"
		
		sed -i 's/<p>Description Begin<\/p>/<p>Description Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Footnote Ending Text from "Footnote End" to "Footnote End."
		
		sed -i 's/<p>Footnote End<\/p>/<p>Footnote End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Footnote Ending Text if not capitalized
		
		sed -i 's/<p>Footnote end.<\/p>/<p>Footnote End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Ending Text from "Secondary Text End" to "Secondary Text End."
		
		sed -i 's/<p>Secondary Text End<\/p>/<p>Secondary Text End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Ending Text if not capitalized
		
		sed -i 's/<p>Secondary text end.<\/p>/<p>Secondary Text End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Ending Text from "Description End" to "Description End."
		
		sed -i 's/<p>Description End<\/p>/<p>Description End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Ending Text if not capitalized
		
		sed -i 's/<p>Description end.<\/p>/<p>Description End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Footnote Ending Text from "End Footnote." to "Footnote End."
		
		sed -i 's/<p>End Footnote.<\/p>/<p>Footnote End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Ending Text from "End Secondary Text." to "Secondary Text End."
		
		sed -i 's/<p>End Secondary Text.<\/p>/<p>Secondary Text End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Description Ending Text from "End Description." to "Description End."
		
		sed -i 's/<p>End Description.<\/p>/<p>Description End.<\/p>/g' ./"$baseName"/"$baseName".html

# Change Footnote Ending Text from "End Footnote" to "Footnote End."

		sed -i 's/<p>End Footnote<\/p>/<p>Footnote End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Secondary Text Ending Text from "End Secondary Text" to "Secondary Text End."
		
		sed -i 's/<p>End Secondary Text<\/p>/<p>Secondary Text End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Figcaption Beginning Text from "Figcaption begin:" to "Figcaption Begin:"
		
		sed -i 's/<p>Figcaption begin:<\/p>/<p>Figcaption Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Figcaption Ending Text from "Figcaption end." to "Figcaption End."
		
		sed -i 's/<p>Figcaption end.<\/p>/<p>Figcaption End.<\/p>/g' ./"$baseName"/"$baseName".html

# Change Figcaption Beginning Text from "Begin Figcaption:" to "Figcaption Begin:"
		
		sed -i 's/<p>Begin Figcaption:<\/p>/<p>Figcaption Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Figcaption Ending Text from "End Figcaption." to "Figcaption End."
		
		sed -i 's/<p>End Figcaption.<\/p>/<p>Figcaption End.<\/p>/g' ./"$baseName"/"$baseName".html

#	New 1.5.2

# Change Caption Beginning Text from "Caption begin:" to "Caption Begin:"
		
		sed -i 's/<p>Caption begin:<\/p>/<p>Caption Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Caption Ending Text from "Caption end." to "Caption End."
		
		sed -i 's/<p>Caption end.<\/p>/<p>Caption End.<\/p>/g' ./"$baseName"/"$baseName".html

# Change Caption Beginning Text from "Begin Caption:" to "Caption Begin:"
		
		sed -i 's/<p>Begin Caption:<\/p>/<p>Caption Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Caption Ending Text from "End Caption." to "Caption End."
		
		sed -i 's/<p>End Caption.<\/p>/<p>Caption End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Transcriber Beginning Text from "Transcriber Note begin:" to "Transcriber Note Begin:"
		
		sed -i 's/<p>Transcriber Note begin:<\/p>/<p>Transcriber Note Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Transcriber Ending Text from "Transcriber Note end." to "Transcriber Note End."
		
		sed -i 's/<p>Transcriber Note end.<\/p>/<p>Transcriber Note End.<\/p>/g' ./"$baseName"/"$baseName".html

# Change Transcriber Beginning Text from "Begin Transcriber Note:" to "Transcriber Note Begin:"
		
		sed -i 's/<p>Begin Transcriber Note:<\/p>/<p>Transcriber Note Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Change Transcriber Ending Text from "End Transcriber Note." to "Transcriber Note End."
		
		sed -i 's/<p>End Transcriber Note.<\/p>/<p>Transcriber Note End.<\/p>/g' ./"$baseName"/"$baseName".html		
		
#	
		
# Change Description Ending Text from "End Description" to "Description End."
		
		sed -i 's/<p>End Description<\/p>/<p>Description End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Add <aside> markup to Footnotes, Secondary Text, and Description text areas (for easier navigation and clarity about regions in a document) #

# Remove superscript if applied to Footnote Begin Text
		
		sed -i 's/<p><sup>Footnote Begin:<\/sup><\/p>/<p>Footnote Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Remove superscript if applied to Footnote End Text
		
		sed -i 's/<p><sup>Footnote End.<\/sup><\/p>/<p>Footnote End.<\/p>/g' ./"$baseName"/"$baseName".html
		
# Remove <strong> element if applied to Footnote End Text
		
		sed -i 's/<p><strong>Footnote End.<\/strong><\/p>/<p>Footnote End.<\/p>/g' ./"$baseName"/"$baseName".html

# Add <aside> element + ARIA role="doc-footnote" to the beginning of a line that has text "Footnote Begin:"

		sed -zi 's/<p>Footnote Begin:<\/p>/<aside role="doc-footnote" class="footnote">\n<p>Footnote Begin:<\/p>/g' ./"$baseName"/"$baseName".html

# Add closing </aside> element to the end of a line that has text "Footnote End."

		sed -zi 's/<p>Footnote End.<\/p>/<p>Footnote End.<\/p>\n<\/aside>/g' ./"$baseName"/"$baseName".html

# New in 1.6.0

# Prevent script error when there is a % sign within <code> and <figcaption>

sed -i 's/\([0-9]\)\(%\)\( \)/\1%% /g' ./"$baseName"/"$baseName".html

#
		
# New 1.5.2

# Transcriber's Notes:

# Replace Transcriber Summary Notes Tags with <div role="banner"> tags and remove all the line breaks in between

		awk '/^<p>Transcriber Summary Begin:<\/p>/{a=1;b="<div role=\"banner\" class=\"transcriber-note-summary\"><p><strong>Transcriber\047s Notes:</strong></p>";next}/^<p>Transcriber Summary End.<\/p>/{a=0;print"</div>";next}a{printf b$0;b="";next}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html

# Replace Transcriber Note with <aside> tag plus an aria-label "Transcriber's Note

		awk '/^<p>Transcriber Note Begin:<\/p>/{a=1;b="<aside class=\"transcriber-note\" aria-label=\"transcriber note\"><p><strong>Transcriber\047s Note:</strong></p>";next}/^<p>Transcriber Note End.<\/p>/{a=0;print"</aside>";next}a{printf b$0;b="";next}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html

# Move transcriber's Summary Notes before the Nav section of the document

		grep "<div role=\"banner\"" ./"$baseName"/"$baseName".html > ./transcriber.txt
		
		
		perl -0777 -pi -e 's/<div role="banner".*\n//g' ./"$baseName"/"$baseName".html
		
		
		awk '/<nav/{system("cat ./transcriber.txt")}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html
		
		rm ./transcriber.txt
	
#

# New 1.5.4

# Replace Figfoot Tags with <aside role="note"> tags and remove all the line breaks in between

awk '/^<p>Figfooter Begin:<\/p>/{a=1;b="<aside role=\"note\" aria-label=\"Figure Footnote\" class=\"figure-footnote\">";next}/^<p>Figfooter End.<\/p>/{a=0;print"</aside></figure>";next}a{printf b$0;b="";next}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html

#		
		
# Remove "Footnote Begin: " text now that this section is wrapped in an <aside> tag with the proper role (doc-footnote) to avoid screen reader output redundancy.

		sed -zi 's/\n<p>Footnote Begin:<\/p>//g' ./"$baseName"/"$baseName".html
		
# Remove "Footnote End." text now that this section is wrapped in an <aside> tag with the proper role (doc-footnote) to avoid screen reader output redundancy.		
	
		sed -zi 's/\n<p>Footnote End.<\/p>//g' ./"$baseName"/"$baseName".html

# Add <aside> element + class="complimentary" to the beginning of a line that has text "Secondary Text Begin:"

		sed -zi 's/<p>Secondary Text Begin:<\/p>/<aside role="complementary" class="complimentary">\n<p>Secondary Text Begin:<\/p>/g' ./"$baseName"/"$baseName".html
		
# Same as above when tags are bold

		sed -zi 's/<p><strong>Secondary Text Begin:<\/strong><\/p>/<aside role="complementary" class="complimentary">\n<p>Secondary Text Begin:<\/p>/g' ./"$baseName"/"$baseName".html

# Add closing </aside> element to the end of a line that has text "Secondary Text End."

		sed -zi 's/<p>Secondary Text End.<\/p>/<p>Secondary Text End.<\/p>\n<\/aside>/g' ./"$baseName"/"$baseName".html

# Same as above when tags are bold

sed -zi 's/<p><strong>Secondary Text End.<\/strong><\/p>/<p>Secondary Text End.<\/p>\n<\/aside>/g' ./"$baseName"/"$baseName".html
		
# Table Markup #

# New 1.5.2

# Remove <tbody> when it immediately follows <table> (Table header not marked in MS Word)

sed -zi 's/<table>\n<tbody>/<table>/g' ./"$baseName"/"$baseName".html

#

# Add class="table-body-start" when tables are marked with header row in MS Word

perl -0777 -pi -e 's/<\/thead>.*\n.*<tbody>.*\n.*<tr class="odd">/<\/thead>\n<tbody>\n<tr class="table-body-start">/g' ./"$baseName"/"$baseName".html

# Add class="table-body-start" when tables are NOT marked with header row in MS Word (table has parent columns with column headers) and previous cell begins with <td>$ (column header)

# Adjusted (was missing </tr> in replace)

perl -0777 -pi -e 's/(<td>\$.*\n).*<\/tr>.*\n.*<tr class="odd">/$1<\/tr>\n<\/thead>\n<tbody>\n<tr class="table-body-start">/g' ./"$baseName"/"$baseName".html

# Adjusted (was missing </tr>) in replace)

perl -0777 -pi -e 's/(<td>\$.*\n).*<\/tr>.*\n.*<tr class="even">/$1<\/tr>\n<\/thead>\n<tbody>\n<tr class="table-body-start">/g' ./"$baseName"/"$baseName".html 

# Add class="table-body-start" when tables are NOT marked with header row in MS Word (table has parent columns with column headers) and previous cell begins with <td>0$ (not a column header)

perl -0777 -pi -e 's/(<td>0\$<\/td>.*\n).*<\/tr>.*\n.*<tr class="odd">/$1<\/tr>\n<\/thead>\n<tbody>\n<tr class="table-body-start">/g' ./"$baseName"/"$baseName".html

# Add class="table-body-start" when tables are NOT marked with header row in MS Word (table has parent columns with column headers) and previous cell is empty - <td><\/td> (left over cells in the row, which occurs when cells are merged in the parent headers)

perl -0777 -pi -e 's/(<td><\/td>.*\n)(.*<\/tr>.*\n)(.*<tr class=.*\n)(.*<td>\d\^ tbody)/$1$2<\/thead>\n<tbody>\n<tr class="table-body-start">\n$4/g' ./"$baseName"/"$baseName".html

#

# Add <th scope="col"> markup (column Headers) to first row in table, which has <th> elements (table header)
		
sed -i 's/<th>/<th scope="col">/g' ./"$baseName"/"$baseName".html

# New in 1.5.8

sed -zi 's/<tr class="even">\n<td><\/td>\n<td><\/td>/<tr class="even">/g' ./"$baseName"/"$baseName".html

sed -zi 's/<tr class="even">\n<td><\/td>/<tr class="even">/g' ./"$baseName"/"$baseName".html

sed -zi 's/<tr class="odd">\n<td><\/td>\n<td><\/td>/<tr class="odd">/g' ./"$baseName"/"$baseName".html

sed -zi 's/<tr class="odd">\n<td><\/td>/<tr class="odd">/g' ./"$baseName"/"$baseName".html

#
		
# Remove CSS class="even" from <tr> elements that Pandoc exports
		
sed -i 's/ class="even"//g' ./"$baseName"/"$baseName".html
		
# Remove CSS class="odd" from <tr> elements that Pandoc exports
		
sed -i 's/ class="odd"//g' ./"$baseName"/"$baseName".html

# New 1.5.2
		
# Place all of the text within Caption Begin...Caption End tags on one line
	
		awk '/^<p>Caption Begin:<\/p>/{a=1;b="<caption>";next}/^<p>Caption End.<\/p>/{a=0;print"</caption>";next}a{printf b$0;b="";next}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html
#		
	
# Move table caption text inside the table element (<caption> is before the <table>

sed -i -e '$!N;s/\(.*<caption>.*\)\(\n\)\(<table>\)/\3\2\1/;t' -e 'P;D' ./"$baseName"/"$baseName".html
		
# Add class to cells at beginning of a row that should NOT be a header cells

sed -zi 's/\(\n<tr>\n\)\(<td>0^ \)/\n<tr class="no-row-header">\n<td>/g' ./"$baseName"/"$baseName".html 

sed -zi 's/\(\n<tr>\n\)\(<td><p>0^ \)/\n<tr class="no-row-header">\n<td><p>/g' ./"$baseName"/"$baseName".html

# Mark tbody in table when it is missing after a table with multiple parent columns and *the first cell in a row that is not a header cells

sed -zi 's/\(<\/tr>\n\)\(<tr>\n\)\(<th scope="row">\)\(tbody\)/\1<\/thead>\n\2<tbody>\n<td>/g' ./"$baseName"/"$baseName".html

# Add Row Headers to first column

sed -zi 's/\(\n<tr>\n\)\(<td>\)/\1<th scope="row">/g' ./"$baseName"/"$baseName".html 

sed -zi 's/\(\n<tr class="table-body-start">\n\)\(<td>\)/\1<th scope="row">/g' ./"$baseName"/"$baseName".html

sed -i 's/\(<th scope="row">.*\)\(<\/td>\)/\1<\/th>/g' ./"$baseName"/"$baseName".html

# New 1.5.2

# Add <tbody> when "tbody" tag is used

perl -0777 -pi -e 's/(<tr class="no-row-header">.*\n)(.*<td>tbody)/<\/thead>\n<tbody>\n<tr class="table-body-start">\n<td>/g' ./"$baseName"/"$baseName".html		
		
# Complex Tables (Irregular Column Headers)

# Create the master column headers (using 2-9 $)

sed -i -e '$!N;s/\(<td>\)\([2-9]\)\($ \)\(.*\)\(<\/td>\)/<th colspan="\2" scope="colgroup">\4<\/th>/' -e 'P;D' ./"$baseName"/"$baseName".html 

# Create the master column headers (using 2-9 $)

sed -i -e '$!N;s/\(<th scope="row">\)\([2-9]\)\($ \)/<th colspan="\2" scope="colgroup">/' -e 'P;D' ./"$baseName"/"$baseName".html 

sed -i -e '$!N;s/\(<th scope="row">\)\($\)/<th scope="col">/' -e 'P;D' ./"$baseName"/"$baseName".html

# Create the master column headers (using 1 $)

sed -i -e '$!N;s/\(<td>\)\(1\)\($ \)\(.*\)\(<\/td>\)/<th scope="col">\4<\/th>/' -e 'P;D' ./"$baseName"/"$baseName".html

# Create the Rowspan for top row (using NUMBER ^)
	
sed -zi 's/\(<th scope="row">\)\([0-9]\)\(\^\)\(@[0-9]\+\)\(<\/th>\)/<td rowspan="\2">\4<\/td>/g' ./"$baseName"/"$baseName".html

# Create the Rowspan for middle rows (using NUMBER ^)
	
sed -zi 's/\(<th scope="row">\)\([1-9]\)\(\^\)/<th rowspan="\2" scope="rowgroup">/g' ./"$baseName"/"$baseName".html

# New 1.6.0

perl -0777 -pi -e 's/(<th scope="row"><p>)(\d+)(\^)(.*\n.*\n.*\n)(.*)(<\/td>)/<th rowspan="$2" scope="rowgroup"><p>$4$5<\/th>/g' ./"$baseName"/"$baseName".html

perl -0777 -pi -e 's/(<th scope="row"><p>)(\d+)(\^)(.*\n.*\n)(.*)(<\/td>)/<th rowspan="$2" scope="rowgroup"><p>$4$5<\/th>/g' ./"$baseName"/"$baseName".html

perl -0777 -pi -e 's/(<th scope="row"><p>)(\d+)(\^)(.*\n)(.*)(<\/td>)/<th rowspan="$2" scope="rowgroup"><p>$4$5<\/th>/g' ./"$baseName"/"$baseName".html

perl -0777 -pi -e 's/(<th scope="row"><p>)(\d+)(\^)(.*\n)(.*)(<\/td>)/<th rowspan="$2" scope="rowgroup"><p>$4$5<\/th>/g' ./"$baseName"/"$baseName".html

# New 1.5.8

# Create the Rowspan for second column in top row (using NUMBER ^)
	
sed -i 's/\(<td>\)\([1-9]\)\(\^\)\(.*\)\(<\/td>\)/<th rowspan="\2" scope="rowgroup">\4<\/th>/g' ./"$baseName"/"$baseName".html


# Create the Rowspan for middle rows (using NUMBER ^) when there are more than 9 rows

sed -zi 's/\(<th scope="row">\)\([1-9][0-9]\)\(\^\)/<th rowspan="\2" scope="rowgroup">/g' ./"$baseName"/"$baseName".html

#


# Create the colspan for the first cell (using 2-9 $)

sed -zi 's/\(<th scope="row">\)\([2-9]\)\(\$\)\(@[0-9]\+\)/<th colspan="\2" scope="colgroup">\4/g' ./"$baseName"/"$baseName".html

# Create the colspan for the first cell (using 1 $)

sed -zi 's/\(<th scope="row">\)\(1\)\(\$\)\(@[0-9]\+\)/<th scope="col">\4/g' ./"$baseName"/"$baseName".html

# Convert cells that have 0 $ with SCOPE=ROW into <td> cells 

sed -zi 's/\(<th scope="row">\)\(0\)\(\$\)\(<\/th>\)/<td>\&nbsp;<\/td>/g' ./"$baseName"/"$baseName".html

## Fixed in 1.5.8 (replace 0$ with EMPTY)

# Convert cells that have 0 $ with SCOPE=COL into <td> cells 

sed -zi 's/\(<th scope="col">\)\(0\)\(\$\)\(<\/th>\)/<td>EMPTY<\/td>/g' ./"$baseName"/"$baseName".html

# Remove 0$ from cells that have <td>0$</td>

sed -zi 's/<td>0$<\/td>/<td>EMPTY<\/td>/g' ./"$baseName"/"$baseName".html

# Remove 0$ from cells that have <td>0$</td>

sed -zi 's/<td>0$<\/td>/<td>\&nbsp;<\/td>/g' ./"$baseName"/"$baseName".html

# Remove the scope=row from the first cell in the table if it has 0$

sed -zi 's/\(<th scope="row">\)\(0\)\(\$\)\(@[0-9]\+\)\(<\/th>\)/<td>\4<\/td>/g' ./"$baseName"/"$baseName".html

# New 1.5.3

# Create the colspan groups in the head of the table (using @1[2-9]) when the first cell has a ROWSPAN

sed -zi 's/\(<tr>\n\)\(<td rowspan="\)\([0-9]\)\(">\)\(\@1\)\([2-9]\)\(<\/td>\)/<col>\n<colgroup span="\6"><\/colgroup>\n\1\2\3\4\7/g' ./"$baseName"/"$baseName".html

#


# Create the colspan groups in the head of the table (using @1[2-9][2-9]) when the first cell has a ROWSPAN

sed -zi 's/\(<tr>\n\)\(<td rowspan="\)\([0-9]\)\(">\)\(\@1\)\([2-9]\)\([2-9]\)\(<\/td>\)/<col>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n\1\2\3\4\8/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @11[2-9]) when the first cell has a ROWSPAN (first two colspans have 1)

sed -zi 's/\(<tr>\n\)\(<td rowspan="\)\([0-9]\)\(">\)\(\@1\)\(1\)\([2-9]\)\(<\/td>\)/<col>\n<col>\n<colgroup span="\7"><\/colgroup>\n\1\2\3\4\8/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @1[2-9]1) when the first cell has a ROWSPAN (first and third colspans have 1)

sed -zi 's/\(<tr>\n\)\(<td rowspan="\)\([0-9]\)\(">\)\(\@1\)\([2-9]\)\(1\)\(<\/td>\)/<col>\n<colgroup span="\6"><\/colgroup>\n<col>\n\1\2\3\4\8/g' ./"$baseName"/"$baseName".html

# New in 1.6.1

# Create the colspan groups in the head of the table (using @1[2-9][2-9][2-9][2-9]) when the first cell has a ROWSPAN (first second and third colspans have [2-9]) AND a word in the first cell

sed -zi 's/\(<tr>\n<th rowspan="\)\([0-9]\)\(" scope="rowgroup">\)\(\@1\)\([2-9]\)\([2-9]\)\([2-9]\)\([2-9]\)\(.*<\/th>\)/<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n<colgroup span="\8"><\/colgroup>\n\1\2\3\9/g' ./"$baseName"/"$baseName".html

# New in 1.5.8

# Create the colspan groups in the head of the table (using @1[2-9][2-9][2-9]) when the first cell has a ROWSPAN (first second and third colspans have [2-9]) AND a word in the first cell

sed -zi 's/\(<tr>\n\)\(<th rowspan="\)\([0-9]\)\(" scope="rowgroup">\)\(\@1\)\([2-9]\)\([2-9]\)\([2-9]\)\(.*<\/th>\)/<col>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n<colgroup span="\8"><\/colgroup>\n\1\2\3\4\9/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @11[2-9][2-9][2-9][2-9]) when the first cell has a ROWSPAN (first and second colspans have [2-9]) AND a word in the first cell

sed -zi 's/\(<tr>\n<th rowspan="\)\([0-9]\)\(" scope="rowgroup">\)\(\@11\)\([2-9]\)\([2-9]\)\([2-9]\)\([2-9]\)\(.*<\/th>\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n<colgroup span="\8"><\/colgroup>\n\1\2\3\9/g' ./"$baseName"/"$baseName".html


# Create the colspan groups in the head of the table (using @11[2-9][2-9][2-9]) when the first cell has a ROWSPAN (first and second colspans have [2-9]) AND a word in the first cell

sed -zi 's/\(<tr>\n<th rowspan="\)\([0-9]\)\(" scope="rowgroup">\)\(\@11\)\([2-9]\)\([2-9]\)\([2-9]\)\(.*<\/th>\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n\1\2\3\8/g' ./"$baseName"/"$baseName".html

#

# Create the colspan groups in the head of the table (using @1[2-9][2-9]) when the first cell has a ROWSPAN (first and second colspans have [2-9]) AND a word in the first cell

sed -zi 's/\(<tr>\n\)\(<th rowspan="\)\([0-9]\)\(" scope="rowgroup">\)\(\@1\)\([2-9]\)\([2-9]\)\(.*<\/th>\)/<col>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n\1\2\3\4\8/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @1[2-9]) when the first cell has a ROWSPAN (a colspan has [2-9]) AND a word in the first cell

sed -zi 's/\(<tr>\n\)\(<th rowspan="\)\([0-9]\)\(" scope="rowgroup">\)\(\@1\)\([2-9]\)\(.*<\/th>\)/<col>\n<colgroup span="\6"><\/colgroup>\n\1\2\3\4\7/g' ./"$baseName"/"$baseName".html

# New in 1.5.8

perl -0777 -pi -e 's/(<tr>.*\n)(<th rowspan=")([0-9])(" scope="rowgroup">)(\@1)([2-9])(.*<\/th>)/<col>\n<colgroup span="$6"><\/colgroup>\n$1$2$3$4$7/g' ./"$baseName"/"$baseName".html

#

# Create the colspan groups in the head of the table (using @[2-9]1[2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th colspan="\)\([2-9]\)\(" scope="colgroup">\)\(\@\)\([2-9]\)\(1\)\([2-9]\)\([2-9]\)/<colgroup span="\6"><\/colgroup>\n<col>\n<colgroup span="\8"><\/colgroup>\n<colgroup span="\9"><\/colgroup>\n\1\2\3\4/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @[2-9]1[2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th colspan="\)\([2-9]\)\(" scope="colgroup">\)\(\@\)\([2-9]\)\(1\)\([2-9]\)/<colgroup span="\6"><\/colgroup>\n<col>\n<colgroup span="\8"><\/colgroup>\n\1\2\3\4/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @1[2-9]1) when the first cell has a COLSPAN (first and third colspans have 1)

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\([2-9]\)\(1\)/<col>\n<colgroup span="\4"><\/colgroup>\n<col>\n\1\2/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @[2-9][2-9]1) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th colspan="\)\([2-9]\)\(" scope="colgroup">\)\(\@\)\([2-9]\)\([2-9]\)\(1\)/<colgroup span="\6">\n<colgroup span="\7"><\/colgroup>\n<col>\1\2\3\4/g' ./"$baseName"/"$baseName".html

# New in 1.5.9

# # Create the colspan groups in the head of the table (using @11[2-9][2-9][2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\(1\)\([2-9]\)\([2-9]\)\([2-9]\)\([2-9]\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n<colgroup span="\8"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# Create the colspan groups in the head of the table (using @11[2-9][2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\(1\)\([2-9]\)\([2-9]\)\([2-9]\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New in 1.6.0

# Create the colspan groups in the head of the table (using @111[2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\(1\)\(1\)\([2-9]\)\([2-9]\)/<col>\n<col>\n<col>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

#


# Create the colspan groups in the head of the table (using @11[2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\(1\)\([2-9]\)\([2-9]\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New in 1.6.0

# Create the colspan groups in the head of the table (using @111[2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@111\)\([2-9]\)/<col>\n<col>\n<col>\n<colgroup span="\4"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

#

#

# Create the colspan groups in the head of the table (using @11[2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\(1\)\([2-9]\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New 1.6.0

# Create the colspan groups in the head of the table (using @1[2-9][2-9][2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\([2-9]\)\([2-9]\)\([2-9]\)\([2-9]\)\(.*\)\(<\/th>\)/<col>\n<colgroup span="\4"><\/colgroup>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n<colgroup span="\7"><\/colgroup>\n\1\2\8<\/th>/g' ./"$baseName"/"$baseName".html

#


# Create the colspan groups in the head of the table (using @1[2-9][2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\([2-9]\)\([2-9]\)\([2-9]\)/<col>\n<colgroup span="\4"><\/colgroup>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New 1.5.3

# Create the colspan groups in the head of the table (using @1[2-9][2-9]) when the first cell has a COLSPAN
 
sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\([2-9]\)\([2-9]\)/<col>\n<colgroup span="\4"><\/colgroup>\n<colgroup span="\5"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html
 
#

# New 1.5.8

# Create the colspan groups in the head of the table (using @1[2-9]) when the first cell has a COLSPAN
 
sed -zi 's/\(<tr>\n\)\(<th scope="col">\)\(\@1\)\([2-9]\)/<col>\n<colgroup span="\4"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html
 
#

# NEW (1.4.9)

# Create the colspan groups in the head of the table (using @[2-9][2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th colspan="[0-9]" scope="colgroup">\)\(\@\)\([2-9]\)\([2-9]\)\([2-9]\)/<colgroup span="\4"><\/colgroup>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New in 1.6.0 
# (Refactor all the code using the <\/td> in the Find what string.

# Add colgroup when first cell is an empty <td> and there are five master columns in this pattern (@111[2-9][2-9])

sed -zi 's/\(<tr>\n\)\(<td>\)\(\@111\)\([2-9]\)\([2-9]\)\(<\/td>\)/<col>\n<col>\n<col>\n<colgroup span="\4"><\/colgroup>\n<colgroup span="\5"><\/colgroup>\n\1\2<\/td>/g' ./"$baseName"/"$baseName".html

#

# New in 1.6.0

# Create the colspan groups in the head of the table (using @[2-9][2-9]) when the first cell has a COLSPAN

sed -zi 's/\(<tr>\n\)\(<th colspan="[0-9]" scope="colgroup">\)\(\@\)\([2-9]\)\([2-9]\)/<colgroup span="\4"><\/colgroup>\n<colgroup span="\5"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

#


# Add colgroup when first cell is an empty <td> and there are four master columns in this pattern (@11[0-9][0-9])

sed -zi 's/\(<tr>\n\)\(<td>\)\(\@1\)\(1\)\([2-9]\)\([2-9]\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n<colgroup span="\6"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New 1.5.2

# Add colgroup when first cell is an empty <td> and there are three master columns in this pattern (@1[0-9][0-9])

sed -zi 's/\(<tr>\n\)\(<td>\)\(\@1\)\(1\)\([2-9]\)/<col>\n<col>\n<colgroup span="\5"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# New 1.6.0

# Add colgroup when first cell is an empty <td> and there are master columns in this pattern (@1[2-9])

sed -zi 's/\(<tr>\n\)\(<td>\)\(\@1\)\([2-9]\)/<col>\n<colgroup span="\4"><\/colgroup>\n\1\2/g' ./"$baseName"/"$baseName".html

# Delete empty first data cell in second row after first master row

sed -zi 's/\(<tr>\)\(\n<th scope="row"><\/th>\)/\1/g' ./"$baseName"/"$baseName".html

##
# Add column headers (<th scope="col">...</th>) in second row when they begin with $ 

sed -i -e '$!N;s/\(<td>\)\($ \)\(.*\)\(<\/td>\)/<th scope="col">\3<\/th>/' -e 'P;D' ./"$baseName"/"$baseName".html

# Add column headers (<th scope="col">...</th>) in first row when they begin with scope=row 

sed -i -e '$!N;s/\(<th scope="row">\)\($ \)\(.*\)\(<\/th>\)/<th scope="col">\3<\/th>/' -e 'P;D' ./"$baseName"/"$baseName".html

sed -i -e '$!N;s/\(<th scope="row">\)\($ \)\(.*\)\(<\/th>\)/<th scope="col">\3<\/th>/' -e 'P;D' ./"$baseName"/"$baseName".html

# Add row headers (<th scope="row">...</th>) in second columns when they begin with ^ 

sed -i -e '$!N;s/\(<td>\)\(\^ \)\(.*\)\(<\/td>\)/<th scope="row">\3<\/th>/' -e 'P;D' ./"$baseName"/"$baseName".html

# New in 1.5.8

sed -i 's/<th scope="row">^ /<th scope="row">/g' ./"$baseName"/"$baseName".html

#

# New in 1.6.0

# Correct row headers if they still have 0^

sed -i 's/\(<th scope="row">0^ \)\(.*\)\(<\/th>\)/<td>\2<\/td>/g' ./"$baseName"/"$baseName".html

# New in 1.6.2

# Correct row headers when they still have 0^ and there are multiple paragraphs in the first cell

sed -i 's/\(<th scope="row"><p>0^ \)\(.*\)\(<\/p>\)/<td><p>\2\3/g' ./"$baseName"/"$baseName".html

#

sed -i 's/<th scope="row">0^<\/th>/<td>\&#160;<\/td>/g' ./"$baseName"/"$baseName".html

#

# Add opening <thead> element at the beginning of first row with master columns when master columns end with closing colgroup tag <\colgroup> tag

sed -zi 's/<\/colgroup>\n<tr>/<\/colgroup>\n<thead>\n<tr>/g' ./"$baseName"/"$baseName".html

# Add opening <thead> element at the beginning of first row with master columns when master columns end with<col> tag

sed -zi 's/<col>\n<tr>/<col>\n<thead>\n<tr>/g' ./"$baseName"/"$baseName".html

## Table Footer Markup ##

# Replace Footer Begin: with <tfoot>

sed -zi 's/\(<\/table>\n\)\(<p>Footer Begin:<\/p>\)/<tfoot><tr>/g' ./"$baseName"/"$baseName".html

# Replace Footer Begin: with <tfoot> when Footer Begin: is bold

sed -zi 's/\(<\/table>\n\)\(<p><strong>Footer Begin:<\/strong><\/p>\)/<tfoot><tr>/g' ./"$baseName"/"$baseName".html

# Replace Footer End with </tfoot>

sed -zi 's/<p>Footer End.<\/p>/<\/td>\n<\/tr><\/tfoot>\n<\/table>/g' ./"$baseName"/"$baseName".html

# Replace Footer End with </tfoot> when Footer End. is bold

sed -zi 's/<p><strong>Footer End.<\/strong><\/p>/<\/td>\n<\/tr><\/tfoot>\n<\/table>/g' ./"$baseName"/"$baseName".html

# Add correct colspan number to table footer based on the NUMBER % in the first line

sed -zi 's/\(<tfoot><tr>\n\)\(<p>\)\([0-9]\+\)\(%\)/\1<td colspan="\3"><p>/g' ./"$baseName"/"$baseName".html

# Add correct colspan number to table footer based on the NUMBER % in the first line when the % and number has <strong> tag

sed -zi 's/\(<tfoot><tr>\n\)\(<p><strong>\)\([0-9]\+\)\(%\)/\1<td colspan="\3"><p><strong>/g' ./"$baseName"/"$baseName".html

# Add accessibility markup for blank table cells

sed -i 's/<td><\/td>/<td>\&nbsp;<\/td>/g' ./"$baseName"/"$baseName".html

# Remove empty cells in a row that has column headers

sed -i '/<td>\&nbsp;<\/td>/N;/^\(.*\)\n\1$/!P; D' ./"$baseName"/"$baseName".html

sed -zi 's/\(<\/th>\)\(\n<td>\&nbsp;<\/td>\)/\1/g' ./"$baseName"/"$baseName".html

sed -i 's/<td>EMPTY<\/td>/<td>\&nbsp;<\/td>/g' ./"$baseName"/"$baseName".html

# New 1.5.2

## Add <tbody> after a row ending with a cell that has "colgroup" and before a row beginning with "rowspan"

perl -0777 -pi -e 's/(scope="colgroup">.*\n)(.*<\/tr>.*\n)(.*<tr>.*\n)(.*<th rowspan)/$1$2<\/thead>\n<tbody>\n<tr class="table-body-start">\n$4/g' ./"$baseName"/"$baseName".html

## Add <tbody> after a row ending with a cell that has "colgroup" and before a row beginning with scope="row"

perl -0777 -pi -e 's/(scope="colgroup">.*\n)(.*<\/tr>.*\n)(.*<tr>.*\n)(.*<th scope="row")/$1$2<\/thead>\n<tbody>\n<tr class="table-body-start">\n$4/g' ./"$baseName"/"$baseName".html

# New in in 1.5.9

## Correct multiple <thead> elements when there are two rowgroups when there are multiple rows that have a rowgroup

perl -0777 -pi -e 's/(scope="colgroup">.*\n)(.*<\/tr>.*\n)(.*<\/thead>.*\n.*<tbody>.*\n<tr class="table-body-start">.*\n)/$1\<\/tr>\n<tr>\n/g' ./"$baseName"/"$baseName".html

#

# New 1.5.4

## Add <tbody> when a table has no <thead> element and the first row starts with a header cell that has scope="row"

perl -0777 -pi -e 's/(<\/caption>.*\n)(.*<tr>.*\n)(.*<th scope="row">)/$1<tbody>\n$2$3/g' ./"$baseName"/"$baseName".html

#

# New in 1.6.0

perl -0777 -pi -e 's/(scope="colgroup">.*\n)(.*<\/tr>.*\n)(.*<tr class="no-row-header">)/$1$2<\/thead>\n<tbody>\n<tr class="table-body-start">/g' ./"$baseName"/"$baseName".html

# perl -0777 -pi -e 's/(scope="colgroup">.*\n)(.*<\/tr>.*\n)(.*<tr>)/$1$2<\/thead>\n<tbody>\n<tr class="table-body-start">/g' ./"$baseName"/EPUB/text/ch001.xhtml

#

# Add ARIA- Label "table Footer" to the <tfoot> element

sed -i 's/<tfoot>/<tfoot aria-label="Table Footer">/g' ./"$baseName"/"$baseName".html

# Make cells with 0$ be a data cell not a header cell.

sed -i 's/<th scope="row"><p>0\$/<td><p>/g' ./"$baseName"/"$baseName".html

# Headings #

# Add tabindex="0" to Heading 6 for easier keyboard navigation by page

sed -i 's/<h6 /<h6 tabindex="0" /g' ./"$baseName"/"$baseName".html

# Lists #

# Remove opening <p> tags inside list items

sed -i 's/<li><p>/<li>/g' ./"$baseName"/"$baseName".html

# Remove closing </p> tags inside list items

sed -i 's/<\/p><\/li>/<\/li>/g' ./"$baseName"/"$baseName".html

# Add missing closing </li> tag from item at the first level of an ordered list

sed -i 's/\(<li>.*\)\(<\/p>\)/\1<\/li>/g' ./"$baseName"/"$baseName".html

# Remove stray closing </li> tag from item at the first level of an ordered list

sed -i 's/<\/ul><\/li>/<\/ul>/g' ./"$baseName"/"$baseName".html

# Remove stray closing </li> tag from the last item of an ordered list

sed -i 's/<\/ol><\/li>/<\/ol>/g' ./"$baseName"/"$baseName".html

# Clean up nested list error: remove closing </li> tag that appears before a new ordered list 

sed -zi 's/\(<\/li>\)\(\n<ol\)/\2/g' ./"$baseName"/"$baseName".html

# Clean up nested list error: remove closing </li> tag that appears before a new unordered list 

sed -zi 's/\(<\/li>\)\(\n<ul\)/\2/g' ./"$baseName"/"$baseName".html

# Add closing </li> tag in a list that has THREE nested ordered lists

sed -zi 's/<\/ol>\n<\/ol>\n<\/ol>\n<\/ol>/<\/ol>\n<\/li>\n<\/ol>\n<\/li>\n<\/ol>\n<\/li>\n<\/ol>/g' ./"$baseName"/"$baseName".html

# Add closing </li> tag in a list that has TWO nested ordered lists

sed -zi 's/<\/ol>\n<\/ol>\n<\/ol>/<\/ol>\n<\/li>\n<\/ol>\n<\/li>\n<\/ol>/g' ./"$baseName"/"$baseName".html

# Add closing </li> tag in a list that has One nested ordered lists

sed -zi 's/<\/ol>\n<\/ol>/<\/ol>\n<\/li>\n<\/ol>/g' ./"$baseName"/"$baseName".html

# New 1.5.6

# Fix nested list error 

sed -zi 's/<\/ol>\n<li>/<\/ol>\n<\/li>\n<li>/g' ./"$baseName"/"$baseName".html

# Add closing </li> tag in a list that has THREE nested unordered lists

sed -zi 's/<\/ul>\n<\/ul>\n<\/ul>\n<\/ul>/<\/ul>\n<\/li>\n<\/ul>\n<\/li>\n<\/ul>\n<\/li>\n<\/ul>/g' ./"$baseName"/"$baseName".html

# Add closing </li> tag in a list that has TWO nested unordered lists

sed -zi 's/<\/ul>\n<\/ul>\n<\/ul>/<\/ul>\n<\/li>\n<\/ul>\n<\/li>\n<\/ul>/g' ./"$baseName"/"$baseName".html

# Add closing </li> tag in a list that has One nested unordered lists

sed -zi 's/<\/ul>\n<\/ul>/<\/ul>\n<\/li>\n<\/ul>/g' ./"$baseName"/"$baseName".html

# Add missing </li> tag in the middle of an nested unordered list

 sed -zi 's/<\/li>\n<\/ul>\n<li>/<\/li>\n<\/ul>\n<\/li>\n<li>/g' ./"$baseName"/"$baseName".html

# Add missing </li> tag in the middle of an nested unordered list when there is one TAB

 sed -zi 's/<\/li>\n<\/ul>\t\n<li>/<\/li>\n<\/ul>\n<\/li>\n<li>/g' ./"$baseName"/"$baseName".html

# Add missing </li> tag in the middle of an nested unordered list when there are two TABs

 sed -zi 's/<\/li>\n<\/ul>\t\t\n<li>/<\/li>\n<\/ul>\n<\/li>\n<li>/g' ./"$baseName"/"$baseName".html

 # Remove spurious </p> tag before a nested ordered list
 sed -zi 's/\(<li>.*\)\(<\/p>\)\(\n<ol\)/\1\3/g' ./"$baseName"/"$baseName".html
 
  # Remove spurious </p> tag before a nested ordered list
 sed -zi 's/\(<li>.*\)\(<\/p>\)\(\n<ul\)/\1\3/g' ./"$baseName"/"$baseName".html
 
 # Add <br> between two paragraphs within one list item
 
 sed -zi 's/<\/li>\n<p>/<br>\n/g' ./"$baseName"/"$baseName".html

# Languages #

# Switch the order of @@@ and <em> element if the <em> element comes before the @@@ tag.

sed -i 's/<em>@@@/@@@<em>/g' ./"$baseName"/"$baseName".html

# Add language markup xml lang="foo" and lang="foo" to text where the main language switches *at the beginning* of text (paragraphs, headings etc.). Use the variable "$language", which is entered manually after the script in the terminal, to enter the proper ISO value

		sed -i 's/>@@@/ lang="'"$language"'" xml:lang="'"$language"'" class="language">/g' ./"$baseName"/"$baseName".html
	
# Add language markup <xml lang="foo" and lang="foo"> to text where the main language switches *in the middle* of a paragraph. Uses the same  variable "$language".	
		
sed -i 's/###/<span lang="'"$language"'" xml:lang="'"$language"'" class="language">/g' ./"$baseName"/"$baseName".html		
		
# Add closing </span> tags for text areas where the secondary language ends in the middle of the paragraph.
		
		sed -i 's/%%%/<\/span>/g' ./"$baseName"/"$baseName".html
		
# Clean Up order of closing </span> tags when the secondary language text also includes formatting.

sed -i 's/<\/span>,<\/em>/<\/em>,<\/span>/g' ./"$baseName"/"$baseName".html

sed -i -E 's/(<em>“)(<span lang=".*">)/\2\1/g' ./"$baseName"/"$baseName".html

sed -i -E 's/(<em>")(<span lang=".*">)/\2\1/g' ./"$baseName"/"$baseName".html

# Add Opening Tags for Fancy Formatting

sed -i 's/~~#/<mark><span lang="'"$language"'" xml:lang="'"$language"'" class="language">/g' ./"$baseName"/"$baseName".html

sed -i 's/~~\$/<mark><span class="dotted">/g' ./"$baseName"/"$baseName".html

sed -i 's/~~^/<mark><span class="dashed">/g' ./"$baseName"/"$baseName".html

sed -i 's/~~+/<mark><span class="doubleunderline">/g' ./"$baseName"/"$baseName".html

sed -i 's/##\$/<span lang="'"$language"'" xml:lang="'"$language"'" class="language" class="dotted">/g' ./"$baseName"/"$baseName".html

sed -i 's/##^/<span lang="'"$language"'" xml:lang="'"$language"'" class="language" class="dashed">/g' ./"$baseName"/"$baseName".html

sed -i 's/##+/<span lang="'"$language"'" xml:lang="'"$language"'" class="language" class="doubleunderline">/g' ./"$baseName"/"$baseName".html

sed -i 's/#~\$/<mark><span lang="'"$language"'" xml:lang="'"$language"'" class="language" class="dotted">/g' ./"$baseName"/"$baseName".html

sed -i 's/#~^/<mark><span lang="'"$language"'" xml:lang="'"$language"'" class="language" class="dashed">/g' ./"$baseName"/"$baseName".html

sed -i 's/#~+/<mark><span lang="'"$language"'" xml:lang="'"$language"'" class="language" class="doubleunderline">/g' ./"$baseName"/"$baseName".html

# Add Closing Tags for Fancy Formatting

sed -i 's/\*\*%/<\/span><\/mark>/g' ./"$baseName"/"$baseName".html

# Remove "Secondary Text Begin:" text

sed -zi 's/\(\n\)\(<p>Secondary Text Begin:<\/p>\)//g' ./"$baseName"/"$baseName".html

# Remove "Secondary Text End." text

sed -zi 's/\(\n\)\(<p>Secondary Text End.<\/p>\)//g' ./"$baseName"/"$baseName".html

# New 1.5.2

# Change <b> to <strong>

sed -i 's/<b>/<strong>/g' ./"$baseName"/"$baseName".html

sed -i 's/<\/b>/<\/strong>/g' ./"$baseName"/"$baseName".html

# Change <i> to <em>

sed -i 's/<i>/<em>/g' ./"$baseName"/"$baseName".html

sed -i 's/<\/i>/<\/em>/g' ./"$baseName"/"$baseName".html

#
	
# Figures #

## Change alternative text that is marked as "decorative" to alt="" (Null-alt) and give it the proper ARIA role (presentation).
		
		sed -i 's/alt="decorative"/alt="" role="presentation"/g' ./"$baseName"/"$baseName".html
		
## Change alternative text that is marked as "Decorative" to alt="" (Null-alt) and give it the proper ARIA role (presentation).
		
		sed -i 's/alt="Decorative"/alt="" role="presentation"/g' ./"$baseName"/"$baseName".html
		
## Change alternative text that is marked as "Decorative" to alt="" (Null-alt) and give it the proper ARIA role (presentation).
		
		sed -i 's/alt="DECORATIVE"/alt="" role="presentation"/g' ./"$baseName"/"$baseName".html
		
## Change alternative text that is marked as alt=" " to alt="" (Null-alt) and give it the proper ARIA role (presentation).
		
		sed -i 's/alt=" "/alt="" role="presentation"/g' ./"$baseName"/"$baseName".html

		
# 1.5.1 Change

awk '/^<p>Figcaption Begin:<\/p>/{a=1;b="<figcaption>";next}/^<p>Figcaption End.<\/p>/{a=0;print"</figcaption>";next}a{printf b$0;b="";next}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html
			
# Replace Figcaption Begin: with <figcaption> tag

sed -i 's/<p>Figcaption Begin:<\/p>/<figcaption>/g' ./"$baseName"/"$baseName".html

# Replace Figcaption End. with </figcaption> tag

sed -i 's/<p>Figcaption End.<\/p>/<\/figcaption>/g' ./"$baseName"/"$baseName".html

# Move <figcaption> onto same line as text

sed -zi 's/<figcaption>\n<p>/<figcaption><p>/g' ./"$baseName"/"$baseName".html

# Move </figcaption> onto same line as text

sed -zi 's/<\/p>\n<\/figcaption>/<\/p><\/figcaption>/g' ./"$baseName"/"$baseName".html

# Move <figcaption> before the <img> tag

sed -i -e '$!N;s/\(<p><img.*>\)\(<\/p>\)\(\n\)\(<figcaption>.*<\/figcaption>\)/\4\n\1\2/;t' -e 'P;D' ./"$baseName"/"$baseName".html

## Add <figure> element around images that no caption

# sed -i -e '$!N;s/\(<p>\)\(<img.* \/>\)\(<\/p>\)/<figure>\2<\/figure>/;t' -e 'P;D' ./"$baseName"/"$baseName".html

perl -i -pe 's/(<p>)(<img.*\/>)(<\/p>)/<figure>$2<\/figure>/g' ./"$baseName"/"$baseName".html 

## Remove <figure> element before image if there is a <figcaption>

sed -i -e '$!N;s/\(<\/figcaption>\n\)\(<figure>\)/\1/;t' -e 'P;D' ./"$baseName"/"$baseName".html

## Add <figure role="group"> element before <figcaption> tag

sed -i 's/<figcaption>/<figure role="group"><figcaption>/g' ./"$baseName"/"$baseName".html

## Extended Descriptions

# Remove </figure> tag if it is followed by a new line that has "Description Begin:" text

sed -zi 's/\(<\/figure>\)\(\n\)\(<p>Description Begin:<\/p>\)/\n\3/g' ./"$baseName"/"$baseName".html

# New 1.6.2

find . -type f -name "*.html" -exec perl -0777 -i -pe 's/(<\/figure>)(\n)(<p>Description Begin:<\/p>)/\n$3/g' {} +

# 

# Replace Description Begin tags with <details> and <summary> tags

sed -zi 's/<p>Description Begin:<\/p>/<details><summary>Extended Description:<\/summary>/g' ./"$baseName"/"$baseName".html

# New 1.6.2

perl -0777 -i -pe 's/<p>Description Begin:<\/p>/<details><summary>Extended Description:<\/summary>/g' ./"$baseName"/"$baseName".html

#

# Remove "Description End." text

sed -zi 's/<p>Description End.<\/p>\n/<\/details><\/figure>\n/g' ./"$baseName"/"$baseName".html

# Catch missing Remove "Description End." text (missing </p>) # New

sed -zi 's/<p>Description End.\n/<\/details>\n<\/figure>\n/g' ./"$baseName"/"$baseName".html

# New 1.5.4

# Remove <figure> closing tag when there are footnotes for figures

perl -0777 -pi -e 's/(<\/details><\/figure>.*\n)(.*<aside role="note")/<\/details>\n$2/g' ./"$baseName"/"$baseName".html

perl -0777 -pi -e 's/(<\/figure>.*\n)(.*<aside role="note")/\n$2/g' ./"$baseName"/"$baseName".html

#

# Math

# Remove spurious backslashes within math equations that have text style (as long as the equations are on their own lines)

sed -i -e :1 -e 's@\(\\text{.*\)\\ \(.*}\)@\1 \2@;t1' ./"$baseName"/"$baseName".html

# Blockquotes:

# New 1.5.5

sed -zi 's/<p>Citation Begin:<\/p>/<footer class="blockquote-footer">/g' ./"$baseName"/"$baseName".html

sed -zi 's/<p>Citation End.<\/p>/<\/footer>\n<\/blockquote>/g' ./"$baseName"/"$baseName".html

sed -zi 's/<p>Quote Begin:<\/p>/<blockquote>/g' ./"$baseName"/"$baseName".html

sed -zi 's/<p>Quote End.<\/p>/<\/blockquote>/g' ./"$baseName"/"$baseName".html

sed -zi 's/\(<\/blockquote>\n\)\(<footer class="blockquote-footer">\)\(\n<blockquote>\)/\2/g' ./"$baseName"/"$baseName".html

sed -zi 's/\(<\/blockquote>\n\)\(<\/footer>\n\)\(<\/blockquote>\)/\2\3/g' ./"$baseName"/"$baseName".html

# Remove spurious blockquotes when "Secondary Text End." tag is indented.

sed -zi 's/\(<blockquote>\)\(\n<\/aside>\n\)\(<\/blockquote>\)/\2/g' ./"$baseName"/"$baseName".html

# Correct path to the images in files
		
find . -type f -name "*.html" -exec sed -i'' -e 's/\(src=".*\)\(\/media\)/src=".\2/g' {} +

# Fix Incorrect Figure Formatting (<figure role="group"> before <figure>)

find . -type f -name "*.html" -exec sed -i -e '$!N;s/\(<figure role="group">\)\(\n\)\(<figure>\)/\1/;t' -e 'P;D' {} +

# Fix Incorrect Figure Formatting (<\figure> before <figcaption>)

find . -type f -name "*.html" -exec sed -i -e '$!N;s/\(<\/figure>\)\(\n\)\(<figcaption>\)/\3/;t' -e 'P;D' {} +

# Fix incorrect closing <\/p> tags after the <img> element

find . -type f -name "*.html" -exec sed -i 's/" \/><\/p><\/p>/" \/><\/p>/g' {} +

# Fix incorrect closing <\/thead> tags after two parent columns rows

# find . -type f -name "*.html" -exec sed -zi 's/\(<th scope="col">.*<\/th>\)\(\n<\/tr>\n\)\(<tr>\n\)\(<th scope="row"\)/\1\2<\/thead>\n<tbody>\n<tr>\n\4/g' {} +

# Clean up Complex tables
#
# Add correct table row header markup when there are multiple paragraphs in cell

sed -i 's/\(<td><p>^ \)\(.*<\/p>\)/<th scope="row"><p>\2/g' ./"$baseName"/"$baseName".html

# Add correct table row header markup when there are multiple paragraphs in cell and there is a missing closing </p> tag

sed -i 's/\(<td><p>^ \)\(.*\)/<th scope="row"><p>\2<\/p>/g' ./"$baseName"/"$baseName".html

# Replace $ tag at the end of a  bulleted list with closing </th> tag

sed -zi 's/\($<\/li>\n\)\(<\/ul><\/td>\)/<\/li>\n<\/ul><\/th>/g' ./"$baseName"/"$baseName".html
#

# Add Skip Links

# Table skip links

# Add skip link before <table> element

sed -i -e '$!N;s/\(<table>\)/<p><a href="#table_#link_to_table#">Skip table<\/a><\/p>\n\1/;t' -e 'P;D' ./"$baseName"/"$baseName".html

# Add skip link target after the </table> element

sed -i -e '$!N;s/\(<\/table>\)/\1\n<a id="table_@link_to_table@" tabindex="-1" aria-label="Table Skipped"><\/a>/;t' -e 'P;D' ./"$baseName"/"$baseName".html

# Add progressive numbers to skip link for tables

awk '{for(x=1;x<=NF;x++)if($x~/#link_to_table#/){sub(/#link_to_table#/,++i)}}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html

# Add progressive numbers to skip link target link for tables

awk '{for(x=1;x<=NF;x++)if($x~/@link_to_table@/){sub(/@link_to_table@/,++i)}}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html

# Add ARIA Labelledby ID (page-"foo") to the page numbers of the document.

	sed -i 's/\(id=\)\("page-[0-9]*"\)/aria-labelledby=\2 \1\2/g' ./"$baseName"/"$baseName".html

# Assign an ARIA labelledby value to all of the Secondary Text and Footnote text areas for each page, for easier navigation and for better screen reader verbosity.

		sed -i '
/aria-labelledby="page-/{
h
s/.*\( aria-labelledby="page-[0-9]*"\).*/\1/
x
s/ aria-labelledby="page-[0-9]*"//
}
/aside role="\(doc-footnote\|complementary\)"/{
G
s/aside\(.*\)\n\(.*\)/aside\2\1/
}' ./"$baseName"/"$baseName".html

# Remove ARIA role of "complementary"

sed -i 's/role="complementary" //g' ./"$baseName"/"$baseName".html

sed -i 's/\(aria-labelledby="\)\(page[0-9]*\)\("\)\( role="doc-footnote"\)/aria-label="\2 footnote region"\4/g' ./"$baseName"/"$baseName".html

# Add <div> for responsive table markup

sed -zi 's/<table>/<div class="table-container" tabindex="0">\n<table>/g' ./"$baseName"/"$baseName".html

sed -zi 's/<\/table>/<\/table>\n<\/div>/g' ./"$baseName"/"$baseName".html

# Clean up missing <p> tags at the beginning of a line

sed -zi 's/\n<strong>/\n<p><strong>/g' ./"$baseName"/"$baseName".html

# Add aria-label and heading to End Notes section when there are footnotes

sed -zi 's/doc-endnotes\">\n<hr \/>/doc-endnotes\">\n<hr aria-label=\"End Notes Section\"\/>\n<h2>End Notes<\/h2>/g' ./"$baseName"/"$baseName".html

# Remove aria role=doc-endnotes from Footnotes section

sed -i 's/<section class=\"footnotes\" role=\"doc-endnotes\">/<section class=\"footnotes\">/g' ./"$baseName"/"$baseName".html

# Add horizontal rule to <aside> element for footnotes

sed -zi 's/<aside class=\"footnote\"/<hr aria-label=\"Extra Content. Press Tab to go to next page\"\/>\n<aside class=\"footnote\"/g' ./"$baseName"/"$baseName".html

# Add horizontal rule to <aside> element for secondary text

sed -zi 's/<aside aria-labelledby/<hr aria-label=\"Extra Content. Press Tab to go to next page\"\/>\n<aside aria-labelledby/g' ./"$baseName"/"$baseName".html

# Remove horizontal rule when there is both a footnotes section and a secondary text section

sed -zi 's/<\/aside>\n<hr aria-label=\"Extra Content. Press Tab to go to next page\"\/>/<\/aside>/g' ./"$baseName"/"$baseName".html

# New 1.5.5

# Remove Horizontal line for Transcriber's notes

sed -zi 's/\(<hr aria-label=\"Extra Content. Press Tab to go to next page\"\/>\n\)\(<aside aria-label="page[0-9]*\)\( Transcriber\)/\2\3/g' ./"$baseName"/"$baseName".html

# Add "Return to text" aria-label for left pointing arrows in Footnotes section

sed -i 's/role=\"doc-backlink\"/aria-label=\"Return to Text\" role=\"doc-backlink\"/g' ./"$baseName"/"$baseName".html

# 1.5.5

# Correct duplicate values for footnote regions when there are two footnote regions on a single page (inside secondary text and inside main body)

perl -0777 -pi -e 's/(<\/aside.*\n.*)(.*<\/aside>.*\n)(<aside aria-label="page\d)/$1$2$3 main body/g' ./"$baseName"/"$baseName".html

# Remove duplicate </figure> tags

sed -i 's/<\/figure><\/figure>/<\/figure>/g' ./"$baseName"/"$baseName".html

# Fill in the blank tags - 2 underscores (middle of a sentence)

sed -i 's/ __ /<span role="math" aria-label="fill in the blank"> _____ <\/span>/g' ./"$baseName"/"$baseName".html

# Fill in the blank tags - 2 underscores (beginning of a sentence)

sed -i 's/<p>__ /<p><span role="math" aria-label="fill in the blank">_____ <\/span>/g' ./"$baseName"/"$baseName".html

# Fill in the blank tags - 2 underscores (end of a sentence)

sed -i 's/ __<\/p>/<span role="math" aria-label="fill in the blank"> _____<\/span><\/p>/g' ./"$baseName"/"$baseName".html

# Fill in the blank tags - 2 underscores (on its own line)

sed -i 's/<p>__<\/p>/<p><span role="math" aria-label="fill in the blank">_____<\/span><\/p>/g' ./"$baseName"/"$baseName".html


# Fill in the answer checkbox

sed -i 's/=== /<span role="math" aria-label="fill in the answer bubble">\&#9633\; <\/span>/g' ./"$baseName"/"$baseName".html

# Fill in the answer bubble

sed -i 's/``` /<span role="math" aria-label="fill in the answer bubble">\&#9675\; <\/span>/g' ./"$baseName"/"$baseName".html

# Version 1.5.6

# Code Blocks

# New in 1.6.2
# Correct percent signs so that script can perform correctly

perl -i -pe 's/%&gt;%/%%&gt;%%/g' ./"$baseName"/"$baseName".html

#
awk '/^<p>Code Begin:<\/p>/{a=1;b="<pre>";next}/^<p>Code End.<\/p>/{a=0;print"</pre>";next}a{printf b$0;b="";next}1' ./"$baseName"/"$baseName".html > tmp && mv tmp ./"$baseName"/"$baseName".html

perl -i -pe 's%(<pre>.*?</pre>)% $1 =~ s/<p>/<code>/gr %eg' ./"$baseName"/"$baseName".html

perl -i -pe 's%(<pre>.*?</pre>)% $1 =~ s/<\/p>/<\/code>/gr %eg' ./"$baseName"/"$baseName".html

sed -zi 's/<code>/\n<code>/g' ./"$baseName"/"$baseName".html

sed -zi 's/<\/code><\/pre>/<\/code>\n<\/pre>/g' ./"$baseName"/"$baseName".html

# New in 1.5.8

sed -i 's/&lt;&lt;&lt;/<code>/g' ./"$baseName"/"$baseName".html

sed -i 's/&gt;&gt;&gt;/<\/code>/g' ./"$baseName"/"$baseName".html

# New in 1.6.2
# restore percent signs after script has processed code correctly
perl -i -pe 's/%%&gt;%%/%&gt;%/g' ./"$baseName"/"$baseName".html

#

# 

# New in 1.6.0

sed -i 's/\([0-9]\)\(%%\)\( \)/\1% /g' ./"$baseName"/"$baseName".html

sed -i 's/\(<td colspan="[1-9]"><p>\)\(% \)/\1/g' ./"$baseName"/"$baseName".html

sed -i 's/\(<td colspan="[1-9][0-9]"><p>\)\(% \)/\1/g' ./"$baseName"/"$baseName".html

#

# New in 1.5.6

# Find lines with missing <p> tags and add them

perl -i -pe 's/(<p>.*[^>])(\n)/$1<\/p>\n/g'  ./"$baseName"/"$baseName".html

# New in 1.5.7

# Remove </p> tags that appear at the end of a display equation opening block

sed -i 's/\(<p><span class="math display">.*\)\(<\/p>\)/\1/g' ./"$baseName"/"$baseName".html

sed -i 's/\(<p><span class="math inline">.*\)\(<\/p>\)/\1/g' ./"$baseName"/"$baseName".html

# New in 1.6.0

# Correct missing </p>

sed -zi 's/\(<p>.*\)\(<\/em>\n\)/\1<\/em><\/p>\n/g' ./"$baseName"/"$baseName".html

#

# New in 1.6.0

# Remove empty hyperlinks

sed -i 's/\(<a href="about:blank">\)\(.*\)\(<\/a>\)/\2/g' ./"$baseName"/"$baseName".html

# New in 1.6.1

sed -i 's/_edited<\/title>/<\/title>/g' ./"$baseName"/"$baseName".html

# Report Log #

# Print the name of the HTML file that was converted as well as the time to the log.txt file.

echo -e "\n"$name".html CONVERTED on "$TIMESTAMP"\n" >> ./Converted-DOCX-HTML/log.txt

# Print the number of pages found in the HTML file to the log.txt file.

grep -c "<h6" ./"$baseName"/"$baseName".html | sed 's/^/Pages: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of heading 1s found in the HTML file to the log.txt file.

grep -c "<h1" ./"$baseName"/"$baseName".html | sed 's/^/Heading 1s: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of heading 2s found in the HTML file to the log.txt file.

grep -c "<h2" ./"$baseName"/"$baseName".html | sed 's/^/Heading 2s: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of heading 3s found in the HTML file to the log.txt file.

grep -c "<h3" ./"$baseName"/"$baseName".html | sed 's/^/Heading 3s: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of heading 4s found in the HTML file to the log.txt file.

grep -c "<h4" ./"$baseName"/"$baseName".html | sed 's/^/Heading 4s: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of images found in the HTML file to the log.txt file.

grep -c "<figure" ./"$baseName"/"$baseName".html | sed 's/^/Images: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of images that have alternative text in the HTML file to the log.txt file.

grep -c -oP 'alt="[^"]+"' ./"$baseName"/"$baseName".html | sed 's/^/Images with Alternative Text: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of images that have captions in the HTML file to the log.txt file.

grep -c "<figcaption>" ./"$baseName"/"$baseName".html | sed 's/^/Images with Captions: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of images the have null alt attribute (alt="") in the HTML file to the log.txt file.

grep -c ""presentation"" ./"$baseName"/"$baseName".html | sed 's/^/Images with Null Alt: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of images that have extended descriptions in the HTML file to the log.txt file.

grep -c "<details>" ./"$baseName"/"$baseName".html | sed 's/^/Extended Descriptions: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of tables in the HTML file to the log.txt file.

grep -c "<table>" ./"$baseName"/"$baseName".html | sed 's/^/Tables: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of tables with captions in the HTML file to the log.txt file.

grep -c "<caption>" ./"$baseName"/"$baseName".html | sed 's/^/Tables with Captions: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of tables with column headers in the HTML file to the log.txt file.

grep -c "<thead>" ./"$baseName"/"$baseName".html | sed 's/^/Tables with Column Headers: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of ordered lists in the HTML file to the log.txt file.

grep -c "<ol" ./"$baseName"/"$baseName".html | sed 's/^/Numbered Lists: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of display equations in the HTML file to the log.txt file.

grep -c "math display" ./"$baseName"/"$baseName".html | sed 's/^/Display Equations: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of inline equations in the HTML file to the log.txt file.

grep -c "math inline" ./"$baseName"/"$baseName".html | sed 's/^/Inline Equations: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of <lang> attributes in the HTML file to the log.txt file.

grep -c "class=\"language\"" ./"$baseName"/"$baseName".html | sed 's/^/Language Tags: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of line numbers (poetry) in the HTML file to the log.txt file.

grep -c "class=\"line-number\"" ./"$baseName"/"$baseName".html | sed 's/^/Line Numbers: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of transcriber's notes in the HTML file to the log.txt file.

grep -c "class=\"transcriber-note\"" ./"$baseName"/"$baseName".html | sed "s/^/Transcriber's Notes: /" >> ./Converted-DOCX-HTML/log.txt

# Print the number of blockquotes

grep -c "<blockquote>" ./"$baseName"/"$baseName".html | sed 's/^/Blockquotes: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of <cite> attributes (citations) in the HTML file to the log.txt file.

grep -c "class=\"blockquote-footer\"" ./"$baseName"/"$baseName".html | sed 's/^/Blockquotes with Citations: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of Secondary Text Areas in the HTML file to the log.txt file.

grep -c "class=\"complimentary\"" ./"$baseName"/"$baseName".html | sed 's/^/Secondary Text Areas: /' >> ./Converted-DOCX-HTML/log.txt

# Print the number of Footnote text areas in the HTML file to the log.txt file.

grep -c "role=\"doc-footnote\"" ./"$baseName"/"$baseName".html | sed 's/^/Footnote Areas: /' >> ./Converted-DOCX-HTML/log.txt

# Remove an empty line at the beginning of the log.txt file

sed -i '1{/^$/d}' ./Converted-DOCX-HTML/log.txt

# Moved below Attention ! Message (see below)

# Print "Done" for completion.

#		echo -e "\n\e[32mDone.\e[0m"
	
# Print "Done" for completion.

		echo -e "\n\e[32mDone.\e[0m"	
		
# Print information to the terminal

echo -e "\nChecking for HTML validation warnings and errors in \e[32m"$name".html\e[0m... "		
		
# Print validation warnings to the terminal

		if 	/c/vnu-runtime-image/bin/vnu.bat --filterpattern ".*Possible misuse of “aria-label.*" ./"$baseName"/"$baseName".html 2>&1 | grep -q 'info warning:' ; then
		
		echo -e "\n\e[33mATTENTION:\e[0m These HTML validation \e[33mwarnings\e[0m were found in \e[32m"$name".html\e[0m:"
		
		/c/vnu-runtime-image/bin/vnu.bat --filterpattern ".*Possible misuse of “aria-label.*" ./"$baseName"/"$baseName".html 2>&1 | sed -n -e 's/^\(.*":\)\(.*\)\(info warning: \)\(.*\)$/\n\2\4/p'
		
		else 
		
		echo -e "\nNo HTML validation \e[33mwarnings\e[0m were found in \e[32m"$name".html\e[0m!";
	fi

# Print validation errors to the terminal

		if 	/c/vnu-runtime-image/bin/vnu.bat --errors-only --filterpattern ".*Duplicate attribute “class”.*" ./"$baseName"/"$baseName".html 2>&1 | grep -q 'error:' ; then
		
		echo -e "\n\e[33mATTENTION:\e[0m These HTML validation \e[31merrors\e[0m were found in \e[32m"$name".html\e[0m:"
		
		/c/vnu-runtime-image/bin/vnu.bat --filterpattern ".*Duplicate attribute “class”.*" ./"$baseName"/"$baseName".html 2>&1 | sed -n -e 's/^\(.*":\)\(.*\)\(error: \)\(.*\)$/\n\2\4/p'
		
		else 
		
		echo -e "\nNo HTML validation \e[31merrors\e[0m were found in \e[32m"$name".html\e[0m!";
	fi

			echo -e "\nOpen \e[32m"$name".html\e[0m in a web browser to perform visual checks."
	
# Search for language switches, mathematical content, and tables in the document. If these are found, print Attention! messages.

	if 	grep -q 'class="language"' ./"$baseName"/"$baseName".html ; then
		
		echo -e "\n\e[33mATTENTION:\e[0m Check the secondary language passages \e[45m\e[30m(purple background)\e[0m\e[0m in \e[32m"$name".html\e[0m and run the \e[35mLangClass- Remove.sh\e[0m Script!"
	fi

	if 	grep -q 'span class="math' ./"$baseName"/"$baseName".html ; then
    
		echo -e "\n\e[33mATTENTION:\e[0m Check the math \e[42m\e[30m(green background)\e[0m\e[0m in \e[32m"$name".html\e[0m!"
	fi

	if 	grep -q '<table>' ./"$baseName"/"$baseName".html ; then
    
		echo -e "\n\e[33mATTENTION:\e[0m Check the table(s) \e[47m\e[30m(headers have a gray background)\e[0m\e[0m in \e[32m"$name".html\e[0m!"
	fi
	
	if 	grep -q '<blockquote>' ./"$baseName"/"$baseName".html ; then
    
		echo -e "\n\e[33mATTENTION:\e[0m Check the blockquote(s) \e[47m\e[30m(indented and gray background)\e[0m\e[0m in \e[32m"$name".html\e[0m!"
	fi
	
	if 	grep -q 'class="line-number"' ./"$baseName"/"$baseName".html ; then
		
		echo -e "\n\e[33mATTENTION:\e[0m Check the line numbers for poetry \e[45m\e[30m(pink background)\e[0m\e[0m in \e[32m"$name".html\e[0m!"
	fi
	
# Move File to Converted DOCX-HTML folder
		
        mv "$x" ./Converted-DOCX-HTML
		
		mv ./Converted-DOCX-HTML/"$x" ./Converted-DOCX-HTML/"$name"_HTML.docx

# Change the name of folders (remove "_edited")
		
		rm -rf ./"$name"/"$name".html
		
		rm -rf ./"$name"/media
		
		mv ./"$baseName"/"$baseName".html ./"$baseName"/"$name".html
		
		if [ ! -d ./"${name%.*}" ]; then

		mkdir "${name%.*}"

		fi
		
		mv ./"$baseName"/"$name".html  ./"$name"/"$name".html
		
		mv ./"$baseName"/media  ./"$name"/media 2> /dev/null
		
		rmdir ./"$baseName"
		
		echo -e "\nA copy of \e[35m"$name".docx\e[0m was moved to the \e[44mConverted-DOCX-HTML\e[0m\e[0m folder."
	
	
done 

# Print information to terminal about the log.txt. file.

		echo -e "\nSee \e[33mlog.txt\e[0m for more details."
