#!/bin/bash

# Joseph Polizzotto
# UC Berkeley
# 510-642-0329
# Version 1.0.6
# Instructions: 1) From a directory containing HTML file(s), open a Git bash window and enter the path to the script. 2) Press ENTER.

for filename in ./*.html; do

        basePath=${filename%.*}
        baseName=${basePath##*/}

# Check if HTML files exist in current working directory. If not, exit the script.

if [[ ! -f "$filename" ]] ; then	
	echo -e "\n\e[31mHTML files are not located in this directory. Exiting...\e[0m"
exit 

fi

# Check if Nu HTML Checker is installed. if not, exit the script.

if [ ! -f  /c/vnu-runtime-image/bin/vnu.bat ]; then
  echo -e "\n\e[31mError: Nu HTML Checker (https://validator.github.io/validator) was not found. Exiting....\e[0m'" >&2
  exit 1
fi

done 

for filename in ./*.html; do

        basePath=${filename%.*}
        baseName=${basePath##*/}

# Print information to the terminal

echo -e "\nChecking \e[32m"$baseName".html\e[0m.... "		
		
# Print validation warnings to the terminal

		if 	/c/vnu-runtime-image/bin/vnu.bat --filterpattern ".*Possible misuse of “aria-label.*" "$baseName".html 2>&1 | grep -q 'info warning:' ; then
		
		echo -e "\n\e[33mATTENTION:\e[0m These HTML validation \e[33mwarnings\e[0m were found in \e[32m"$baseName".html\e[0m:"
		
		/c/vnu-runtime-image/bin/vnu.bat --filterpattern ".*Possible misuse of “aria-label.*" "$baseName".html 2>&1 | sed -n -e 's/^\(.*":\)\(.*\)\(info warning: \)\(.*\)$/\n\2\4/p'
		
		else 
		
		echo -e "\nNo HTML validation \e[33mwarnings\e[0m were found in \e[32m"$baseName".html\e[0m!";
	fi

# Print validation errors to the terminal

		if 	/c/vnu-runtime-image/bin/vnu.bat --errors-only --filterpattern ".*Duplicate attribute “class”.*" "$baseName".html 2>&1 | grep -q 'error:' ; then
		
		echo -e "\n\e[33mATTENTION:\e[0m These HTML validation \e[31merrors\e[0m were found in \e[32m"$baseName".html\e[0m:"
		
		/c/vnu-runtime-image/bin/vnu.bat --filterpattern ".*Duplicate attribute “class”.*" "$baseName".html 2>&1 | sed -n -e 's/^\(.*":\)\(.*\)\(error: \)\(.*\)$/\n\2\4/p'
		
		else 
		
		echo -e "\nNo HTML validation \e[31merrors\e[0m were found in \e[32m"$baseName".html\e[0m!";
	fi
	
done